static const void * const LABELS[] = {
        &&MVMDispOpcodeStartResumption,
        &&MVMDispOpcodeNextResumption,
        &&MVMDispOpcodeResumeTopmost,
        &&MVMDispOpcodeResumeCaller,
        &&MVMDispOpcodeGuardResumeInitCallsite,
        &&MVMDispOpcodeGuardNoResumptionTopmost,
        &&MVMDispOpcodeGuardNoResumptionCaller,
        &&MVMDispOpcodeUpdateResumeState,
        &&MVMDispOpcodeGuardArgType,
        &&MVMDispOpcodeGuardArgTypeConc,
        &&MVMDispOpcodeGuardArgTypeTypeObject,
        &&MVMDispOpcodeGuardArgConc,
        &&MVMDispOpcodeGuardArgTypeObject,
        &&MVMDispOpcodeGuardArgLiteralObj,
        &&MVMDispOpcodeGuardArgLiteralStr,
        &&MVMDispOpcodeGuardArgLiteralInt,
        &&MVMDispOpcodeGuardArgLiteralNum,
        &&MVMDispOpcodeGuardArgNotLiteralObj,
        &&MVMDispOpcodeGuardArgHLL,
        &&MVMDispOpcodeGuardTempType,
        &&MVMDispOpcodeGuardTempTypeConc,
        &&MVMDispOpcodeGuardTempTypeTypeObject,
        &&MVMDispOpcodeGuardTempConc,
        &&MVMDispOpcodeGuardTempTypeObject,
        &&MVMDispOpcodeGuardTempLiteralObj,
        &&MVMDispOpcodeGuardTempLiteralStr,
        &&MVMDispOpcodeGuardTempLiteralInt,
        &&MVMDispOpcodeGuardTempLiteralNum,
        &&MVMDispOpcodeGuardTempNotLiteralObj,
        &&MVMDispOpcodeGuardTempHLL,
        &&MVMDispOpcodeLoadCaptureValue,
        &&MVMDispOpcodeLoadResumeInitValue,
        &&MVMDispOpcodeLoadResumeState,
        &&MVMDispOpcodeLoadConstantObjOrStr,
        &&MVMDispOpcodeLoadConstantInt,
        &&MVMDispOpcodeLoadConstantNum,
        &&MVMDispOpcodeLoadAttributeObj,
        &&MVMDispOpcodeLoadAttributeInt,
        &&MVMDispOpcodeLoadAttributeNum,
        &&MVMDispOpcodeLoadAttributeStr,
        &&MVMDispOpcodeLoadHOW,
        &&MVMDispOpcodeUnboxInt,
        &&MVMDispOpcodeUnboxNum,
        &&MVMDispOpcodeUnboxStr,
        &&MVMDispOpcodeLookup,
        &&MVMDispOpcodeSet,
        &&MVMDispOpcodeResultValueObj,
        &&MVMDispOpcodeResultValueStr,
        &&MVMDispOpcodeResultValueInt,
        &&MVMDispOpcodeResultValueNum,
        &&MVMDispOpcodeBindFailureToResumption,
        &&MVMDispOpcodeBindCompletionToResumption,
        &&MVMDispOpcodeUseArgsTail,
        &&MVMDispOpcodeCopyArgsTail,
        &&MVMDispOpcodeResultBytecode,
        &&MVMDispOpcodeResultCFunction,
        &&MVMDispOpcodeResultForeignCode
};
