/* This file is generated from src/core/oplist by tools/update_ops.raku. */

static const void * const LABELS[] = {
    &&OP_no_op,
    &&OP_const_i8,
    &&OP_const_i16,
    &&OP_const_i32,
    &&OP_const_i64,
    &&OP_const_n32,
    &&OP_const_n64,
    &&OP_const_s,
    &&OP_set,
    &&OP_extend_u8,
    &&OP_extend_u16,
    &&OP_extend_u32,
    &&OP_extend_i8,
    &&OP_extend_i16,
    &&OP_extend_i32,
    &&OP_trunc_u8,
    &&OP_trunc_u16,
    &&OP_trunc_u32,
    &&OP_trunc_i8,
    &&OP_trunc_i16,
    &&OP_trunc_i32,
    &&OP_extend_n32,
    &&OP_trunc_n32,
    &&OP_goto,
    &&OP_if_i,
    &&OP_unless_i,
    &&OP_if_n,
    &&OP_unless_n,
    &&OP_if_s,
    &&OP_unless_s,
    &&OP_if_s0,
    &&OP_unless_s0,
    &&OP_DEPRECATED_62,
    &&OP_DEPRECATED_63,
    &&OP_jumplist,
    &&OP_getlex,
    &&OP_bindlex,
    &&OP_getlex_ni,
    &&OP_getlex_nn,
    &&OP_getlex_ns,
    &&OP_getlex_no,
    &&OP_bindlex_ni,
    &&OP_bindlex_nn,
    &&OP_bindlex_ns,
    &&OP_bindlex_no,
    &&OP_getlex_ng,
    &&OP_bindlex_ng,
    &&OP_getdynlex,
    &&OP_binddynlex,
    &&OP_setlexvalue,
    &&OP_lexprimspec,
    &&OP_return_i,
    &&OP_return_n,
    &&OP_return_s,
    &&OP_return_o,
    &&OP_return,
    &&OP_eq_i,
    &&OP_ne_i,
    &&OP_lt_i,
    &&OP_le_i,
    &&OP_gt_i,
    &&OP_ge_i,
    &&OP_cmp_i,
    &&OP_add_i,
    &&OP_sub_i,
    &&OP_mul_i,
    &&OP_div_i,
    &&OP_div_u,
    &&OP_mod_i,
    &&OP_mod_u,
    &&OP_neg_i,
    &&OP_abs_i,
    &&OP_inc_i,
    &&OP_inc_u,
    &&OP_dec_i,
    &&OP_dec_u,
    &&OP_band_i,
    &&OP_bor_i,
    &&OP_bxor_i,
    &&OP_bnot_i,
    &&OP_blshift_i,
    &&OP_brshift_i,
    &&OP_pow_i,
    &&OP_not_i,
    &&OP_gcd_i,
    &&OP_lcm_i,
    &&OP_eq_n,
    &&OP_ne_n,
    &&OP_lt_n,
    &&OP_le_n,
    &&OP_gt_n,
    &&OP_ge_n,
    &&OP_cmp_n,
    &&OP_add_n,
    &&OP_sub_n,
    &&OP_mul_n,
    &&OP_div_n,
    &&OP_mod_n,
    &&OP_neg_n,
    &&OP_abs_n,
    &&OP_pow_n,
    &&OP_ceil_n,
    &&OP_floor_n,
    &&OP_sin_n,
    &&OP_asin_n,
    &&OP_cos_n,
    &&OP_acos_n,
    &&OP_tan_n,
    &&OP_atan_n,
    &&OP_atan2_n,
    &&OP_DEPRECATED_35,
    &&OP_DEPRECATED_36,
    &&OP_sinh_n,
    &&OP_cosh_n,
    &&OP_tanh_n,
    &&OP_DEPRECATED_37,
    &&OP_sqrt_n,
    &&OP_log_n,
    &&OP_exp_n,
    &&OP_coerce_in,
    &&OP_coerce_ni,
    &&OP_coerce_is,
    &&OP_coerce_ns,
    &&OP_coerce_si,
    &&OP_coerce_sn,
    &&OP_DEPRECATED_66,
    &&OP_DEPRECATED_67,
    &&OP_DEPRECATED_99,
    &&OP_DEPRECATED_100,
    &&OP_DEPRECATED_101,
    &&OP_DEPRECATED_102,
    &&OP_DEPRECATED_103,
    &&OP_DEPRECATED_104,
    &&OP_DEPRECATED_105,
    &&OP_DEPRECATED_106,
    &&OP_DEPRECATED_70,
    &&OP_DEPRECATED_71,
    &&OP_DEPRECATED_72,
    &&OP_DEPRECATED_73,
    &&OP_DEPRECATED_74,
    &&OP_checkarity,
    &&OP_param_rp_i,
    &&OP_param_rp_n,
    &&OP_param_rp_s,
    &&OP_param_rp_o,
    &&OP_param_op_i,
    &&OP_param_op_n,
    &&OP_param_op_s,
    &&OP_param_op_o,
    &&OP_param_rn_i,
    &&OP_param_rn_n,
    &&OP_param_rn_s,
    &&OP_param_rn_o,
    &&OP_param_on_i,
    &&OP_param_on_n,
    &&OP_param_on_s,
    &&OP_param_on_o,
    &&OP_param_sp,
    &&OP_param_sn,
    &&OP_getcode,
    &&OP_caller,
    &&OP_capturelex,
    &&OP_takeclosure,
    &&OP_exception,
    &&OP_bindexmessage,
    &&OP_bindexpayload,
    &&OP_bindexcategory,
    &&OP_getexmessage,
    &&OP_getexpayload,
    &&OP_getexcategory,
    &&OP_throwdyn,
    &&OP_throwlex,
    &&OP_throwlexotic,
    &&OP_throwcatdyn,
    &&OP_throwcatlex,
    &&OP_throwcatlexotic,
    &&OP_die,
    &&OP_rethrow,
    &&OP_resume,
    &&OP_takehandlerresult,
    &&OP_DEPRECATED_33,
    &&OP_DEPRECATED_34,
    &&OP_backtracestrings,
    &&OP_usecapture,
    &&OP_savecapture,
    &&OP_captureposelems,
    &&OP_captureposarg,
    &&OP_captureposarg_i,
    &&OP_captureposarg_n,
    &&OP_captureposarg_s,
    &&OP_captureposprimspec,
    &&OP_captureexistsnamed,
    &&OP_capturehasnameds,
    &&OP_DEPRECATED_69,
    &&OP_DEPRECATED_60,
    &&OP_DEPRECATED_61,
    &&OP_null_s,
    &&OP_isnull_s,
    &&OP_eq_s,
    &&OP_ne_s,
    &&OP_gt_s,
    &&OP_ge_s,
    &&OP_lt_s,
    &&OP_le_s,
    &&OP_cmp_s,
    &&OP_eqat_s,
    &&OP_eqatic_s,
    &&OP_haveat_s,
    &&OP_concat_s,
    &&OP_repeat_s,
    &&OP_substr_s,
    &&OP_index_s,
    &&OP_DEPRECATED_40,
    &&OP_codes_s,
    &&OP_getcp_s,
    &&OP_indexcp_s,
    &&OP_uc,
    &&OP_lc,
    &&OP_tc,
    &&OP_split,
    &&OP_join,
    &&OP_getcpbyname,
    &&OP_indexat,
    &&OP_indexnat,
    &&OP_unipropcode,
    &&OP_unipvalcode,
    &&OP_hasuniprop,
    &&OP_hasunipropc,
    &&OP_chars,
    &&OP_chr,
    &&OP_ordfirst,
    &&OP_ordat,
    &&OP_rindexfrom,
    &&OP_escape,
    &&OP_flip,
    &&OP_setbuffersize_fh,
    &&OP_iscclass,
    &&OP_findcclass,
    &&OP_findnotcclass,
    &&OP_nfafromstatelist,
    &&OP_nfarunproto,
    &&OP_nfarunalt,
    &&OP_radix,
    &&OP_encode,
    &&OP_decode,
    &&OP_istrue_s,
    &&OP_isfalse_s,
    &&OP_null,
    &&OP_isnull,
    &&OP_ifnonnull,
    &&OP_DEPRECATED_75,
    &&OP_DEPRECATED_76,
    &&OP_DEPRECATED_77,
    &&OP_DEPRECATED_78,
    &&OP_create,
    &&OP_clone,
    &&OP_isconcrete,
    &&OP_rebless,
    &&OP_istype,
    &&OP_objprimspec,
    &&OP_gethow,
    &&OP_getwhat,
    &&OP_getwho,
    &&OP_setwho,
    &&OP_reprname,
    &&OP_getwhere,
    &&OP_eqaddr,
    &&OP_bindattr_i,
    &&OP_bindattr_n,
    &&OP_bindattr_s,
    &&OP_bindattr_o,
    &&OP_bindattrs_i,
    &&OP_bindattrs_n,
    &&OP_bindattrs_s,
    &&OP_bindattrs_o,
    &&OP_getattr_i,
    &&OP_getattr_n,
    &&OP_getattr_s,
    &&OP_getattr_o,
    &&OP_getattrs_i,
    &&OP_getattrs_n,
    &&OP_getattrs_s,
    &&OP_getattrs_o,
    &&OP_attrinited,
    &&OP_box_i,
    &&OP_box_n,
    &&OP_box_s,
    &&OP_unbox_i,
    &&OP_unbox_n,
    &&OP_unbox_s,
    &&OP_atpos_i,
    &&OP_atpos_n,
    &&OP_atpos_s,
    &&OP_atpos_o,
    &&OP_bindpos_i,
    &&OP_bindpos_n,
    &&OP_bindpos_s,
    &&OP_bindpos_o,
    &&OP_push_i,
    &&OP_push_n,
    &&OP_push_s,
    &&OP_push_o,
    &&OP_pop_i,
    &&OP_pop_n,
    &&OP_pop_s,
    &&OP_pop_o,
    &&OP_shift_i,
    &&OP_shift_n,
    &&OP_shift_s,
    &&OP_shift_o,
    &&OP_unshift_i,
    &&OP_unshift_n,
    &&OP_unshift_s,
    &&OP_unshift_o,
    &&OP_splice,
    &&OP_setelemspos,
    &&OP_existspos,
    &&OP_atkey_i,
    &&OP_atkey_n,
    &&OP_atkey_s,
    &&OP_atkey_o,
    &&OP_bindkey_i,
    &&OP_bindkey_n,
    &&OP_bindkey_s,
    &&OP_bindkey_o,
    &&OP_existskey,
    &&OP_deletekey,
    &&OP_elems,
    &&OP_knowhow,
    &&OP_knowhowattr,
    &&OP_newtype,
    &&OP_composetype,
    &&OP_DEPRECATED_83,
    &&OP_DEPRECATED_84,
    &&OP_settypecache,
    &&OP_settypecheckmode,
    &&OP_setboolspec,
    &&OP_DEPRECATED_64,
    &&OP_DEPRECATED_65,
    &&OP_bootint,
    &&OP_bootnum,
    &&OP_bootstr,
    &&OP_bootarray,
    &&OP_bootintarray,
    &&OP_bootnumarray,
    &&OP_bootstrarray,
    &&OP_boothash,
    &&OP_isint,
    &&OP_isnum,
    &&OP_isstr,
    &&OP_islist,
    &&OP_ishash,
    &&OP_sethllconfig,
    &&OP_hllboxtype_i,
    &&OP_hllboxtype_n,
    &&OP_hllboxtype_s,
    &&OP_hlllist,
    &&OP_hllhash,
    &&OP_getcomp,
    &&OP_bindcomp,
    &&OP_getcurhllsym,
    &&OP_bindcurhllsym,
    &&OP_gethllsym,
    &&OP_bindhllsym,
    &&OP_settypehll,
    &&OP_settypehllrole,
    &&OP_DEPRECATED_95,
    &&OP_DEPRECATED_96,
    &&OP_usecompileehllconfig,
    &&OP_usecompilerhllconfig,
    &&OP_iter,
    &&OP_iterkey_s,
    &&OP_iterval,
    &&OP_getcodename,
    &&OP_iscoderef,
    &&OP_getcodeobj,
    &&OP_setcodeobj,
    &&OP_setcodename,
    &&OP_forceouterctx,
    &&OP_DEPRECATED_97,
    &&OP_DEPRECATED_98,
    &&OP_freshcoderef,
    &&OP_markcodestatic,
    &&OP_markcodestub,
    &&OP_getstaticcode,
    &&OP_getcodecuid,
    &&OP_DEPRECATED_90,
    &&OP_DEPRECATED_91,
    &&OP_assign,
    &&OP_assignunchecked,
    &&OP_iscont,
    &&OP_decont,
    &&OP_setcontspec,
    &&OP_sha1,
    &&OP_createsc,
    &&OP_scsetobj,
    &&OP_scsetcode,
    &&OP_scgetobj,
    &&OP_scgethandle,
    &&OP_scgetobjidx,
    &&OP_scsetdesc,
    &&OP_scobjcount,
    &&OP_setobjsc,
    &&OP_getobjsc,
    &&OP_serialize,
    &&OP_deserialize,
    &&OP_wval,
    &&OP_wval_wide,
    &&OP_scwbdisable,
    &&OP_scwbenable,
    &&OP_pushcompsc,
    &&OP_popcompsc,
    &&OP_scgetdesc,
    &&OP_loadbytecode,
    &&OP_newmixintype,
    &&OP_installconfprog,
    &&OP_iscompunit,
    &&OP_compunitmainline,
    &&OP_compunitcodes,
    &&OP_ctx,
    &&OP_ctxouter,
    &&OP_ctxcaller,
    &&OP_ctxlexpad,
    &&OP_curcode,
    &&OP_callercode,
    &&OP_add_I,
    &&OP_sub_I,
    &&OP_mul_I,
    &&OP_div_I,
    &&OP_mod_I,
    &&OP_neg_I,
    &&OP_abs_I,
    &&OP_cmp_I,
    &&OP_eq_I,
    &&OP_ne_I,
    &&OP_lt_I,
    &&OP_le_I,
    &&OP_gt_I,
    &&OP_ge_I,
    &&OP_bor_I,
    &&OP_bxor_I,
    &&OP_band_I,
    &&OP_bnot_I,
    &&OP_blshift_I,
    &&OP_brshift_I,
    &&OP_pow_I,
    &&OP_gcd_I,
    &&OP_lcm_I,
    &&OP_expmod_I,
    &&OP_isprime_I,
    &&OP_rand_I,
    &&OP_coerce_In,
    &&OP_coerce_Is,
    &&OP_coerce_nI,
    &&OP_coerce_sI,
    &&OP_isbig_I,
    &&OP_bool_I,
    &&OP_base_I,
    &&OP_radix_I,
    &&OP_div_In,
    &&OP_copy_f,
    &&OP_append_f,
    &&OP_rename_f,
    &&OP_delete_f,
    &&OP_chmod_f,
    &&OP_exists_f,
    &&OP_mkdir,
    &&OP_rmdir,
    &&OP_open_dir,
    &&OP_read_dir,
    &&OP_close_dir,
    &&OP_open_fh,
    &&OP_close_fh,
    &&OP_DEPRECATED_23,
    &&OP_DEPRECATED_27,
    &&OP_DEPRECATED_28,
    &&OP_DEPRECATED_18,
    &&OP_seek_fh,
    &&OP_lock_fh,
    &&OP_unlock_fh,
    &&OP_sync_fh,
    &&OP_trunc_fh,
    &&OP_eof_fh,
    &&OP_getstdin,
    &&OP_getstdout,
    &&OP_getstderr,
    &&OP_connect_sk,
    &&OP_socket,
    &&OP_bind_sk,
    &&OP_DEPRECATED_24,
    &&OP_accept_sk,
    &&OP_decodetocodes,
    &&OP_encodefromcodes,
    &&OP_DEPRECATED_17,
    &&OP_print,
    &&OP_say,
    &&OP_DEPRECATED_22,
    &&OP_tell_fh,
    &&OP_stat,
    &&OP_DEPRECATED_79,
    &&OP_DEPRECATED_80,
    &&OP_chdir,
    &&OP_srand,
    &&OP_rand_i,
    &&OP_rand_n,
    &&OP_DEPRECATED_46,
    &&OP_sleep,
    &&OP_newthread,
    &&OP_threadjoin,
    &&OP_DEPRECATED_47,
    &&OP_exit,
    &&OP_DEPRECATED_30,
    &&OP_cwd,
    &&OP_clargs,
    &&OP_getenvhash,
    &&OP_loadlib,
    &&OP_freelib,
    &&OP_findsym,
    &&OP_dropsym,
    &&OP_loadext,
    &&OP_backendconfig,
    &&OP_getlexouter,
    &&OP_getlexrel,
    &&OP_getlexreldyn,
    &&OP_getlexrelcaller,
    &&OP_getlexcaller,
    &&OP_bitand_s,
    &&OP_bitor_s,
    &&OP_bitxor_s,
    &&OP_isnanorinf,
    &&OP_inf,
    &&OP_neginf,
    &&OP_nan,
    &&OP_getpid,
    &&OP_DEPRECATED_29,
    &&OP_filereadable,
    &&OP_filewritable,
    &&OP_fileexecutable,
    &&OP_DEPRECATED_19,
    &&OP_capturenamedshash,
    &&OP_read_fhb,
    &&OP_write_fhb,
    &&OP_replace,
    &&OP_newexception,
    &&OP_permit,
    &&OP_backtrace,
    &&OP_symlink,
    &&OP_link,
    &&OP_gethostname,
    &&OP_exreturnafterunwind,
    &&OP_vmeventsubscribe,
    &&OP_continuationreset,
    &&OP_continuationcontrol,
    &&OP_continuationinvoke,
    &&OP_randscale_n,
    &&OP_uniisblock,
    &&OP_assertparamcheck,
    &&OP_hintfor,
    &&OP_paramnamesused,
    &&OP_getuniname,
    &&OP_getuniprop_int,
    &&OP_getuniprop_bool,
    &&OP_getuniprop_str,
    &&OP_matchuniprop,
    &&OP_nativecallbuild,
    &&OP_nativecallinvoke,
    &&OP_nativecallrefresh,
    &&OP_threadrun,
    &&OP_threadid,
    &&OP_threadyield,
    &&OP_currentthread,
    &&OP_lock,
    &&OP_unlock,
    &&OP_semacquire,
    &&OP_semtryacquire,
    &&OP_semrelease,
    &&OP_getlockcondvar,
    &&OP_condwait,
    &&OP_condsignalone,
    &&OP_condsignalall,
    &&OP_queuepoll,
    &&OP_DEPRECATED_82,
    &&OP_ctxouterskipthunks,
    &&OP_ctxcallerskipthunks,
    &&OP_timer,
    &&OP_cancel,
    &&OP_signal,
    &&OP_watchfile,
    &&OP_asyncconnect,
    &&OP_asynclisten,
    &&OP_DEPRECATED_14,
    &&OP_asyncwritebytes,
    &&OP_DEPRECATED_16,
    &&OP_asyncreadbytes,
    &&OP_getlexstatic_o,
    &&OP_getlexperinvtype_o,
    &&OP_execname,
    &&OP_const_i64_16,
    &&OP_const_i64_32,
    &&OP_isnonnull,
    &&OP_param_rn2_i,
    &&OP_param_rn2_n,
    &&OP_param_rn2_s,
    &&OP_param_rn2_o,
    &&OP_param_on2_i,
    &&OP_param_on2_n,
    &&OP_param_on2_s,
    &&OP_param_on2_o,
    &&OP_osrpoint,
    &&OP_nativecallcast,
    &&OP_spawnprocasync,
    &&OP_killprocasync,
    &&OP_startprofile,
    &&OP_endprofile,
    &&OP_objectid,
    &&OP_settypefinalize,
    &&OP_force_gc,
    &&OP_nativecallglobal,
    &&OP_DEPRECATED_32,
    &&OP_setparameterizer,
    &&OP_parameterizetype,
    &&OP_typeparameterized,
    &&OP_typeparameters,
    &&OP_typeparameterat,
    &&OP_readlink,
    &&OP_lstat,
    &&OP_iscont_i,
    &&OP_iscont_n,
    &&OP_iscont_s,
    &&OP_assign_i,
    &&OP_assign_n,
    &&OP_assign_s,
    &&OP_decont_i,
    &&OP_decont_n,
    &&OP_decont_s,
    &&OP_getrusage,
    &&OP_threadlockcount,
    &&OP_bindcomplete,
    &&OP_getlexref_i,
    &&OP_getlexref_n,
    &&OP_getlexref_s,
    &&OP_getlexref_ni,
    &&OP_getlexref_nn,
    &&OP_getlexref_ns,
    &&OP_atposref_i,
    &&OP_atposref_n,
    &&OP_atposref_s,
    &&OP_getattrref_i,
    &&OP_getattrref_n,
    &&OP_getattrref_s,
    &&OP_getattrsref_i,
    &&OP_getattrsref_n,
    &&OP_getattrsref_s,
    &&OP_nativecallsizeof,
    &&OP_encodenorm,
    &&OP_normalizecodes,
    &&OP_strfromcodes,
    &&OP_strtocodes,
    &&OP_getcodelocation,
    &&OP_eqatim_s,
    &&OP_ordbaseat,
    &&OP_neverrepossess,
    &&OP_scdisclaim,
    &&OP_setthreadname,
    &&OP_atpos2d_i,
    &&OP_atpos2d_n,
    &&OP_atpos2d_s,
    &&OP_atpos2d_o,
    &&OP_atpos3d_i,
    &&OP_atpos3d_n,
    &&OP_atpos3d_s,
    &&OP_atpos3d_o,
    &&OP_atposnd_i,
    &&OP_atposnd_n,
    &&OP_atposnd_s,
    &&OP_atposnd_o,
    &&OP_bindpos2d_i,
    &&OP_bindpos2d_n,
    &&OP_bindpos2d_s,
    &&OP_bindpos2d_o,
    &&OP_bindpos3d_i,
    &&OP_bindpos3d_n,
    &&OP_bindpos3d_s,
    &&OP_bindpos3d_o,
    &&OP_bindposnd_i,
    &&OP_bindposnd_n,
    &&OP_bindposnd_s,
    &&OP_bindposnd_o,
    &&OP_dimensions,
    &&OP_setdimensions,
    &&OP_numdimensions,
    &&OP_ctxcode,
    &&OP_isrwcont,
    &&OP_fc,
    &&OP_DEPRECATED_25,
    &&OP_DEPRECATED_21,
    &&OP_encoderep,
    &&OP_istty_fh,
    &&OP_multidimref_i,
    &&OP_multidimref_n,
    &&OP_multidimref_s,
    &&OP_fileno_fh,
    &&OP_asyncudp,
    &&OP_DEPRECATED_15,
    &&OP_asyncwritebytesto,
    &&OP_objprimbits,
    &&OP_objprimunsigned,
    &&OP_ctxnt,
    &&OP_DEPRECATED_6,
    &&OP_DEPRECATED_7,
    &&OP_DEPRECATED_8,
    &&OP_getlexref_i32,
    &&OP_getlexref_i16,
    &&OP_getlexref_i8,
    &&OP_getlexref_n32,
    &&OP_box_u,
    &&OP_unbox_u,
    &&OP_coerce_iu,
    &&OP_coerce_ui,
    &&OP_coerce_nu,
    &&OP_coerce_un,
    &&OP_decont_u,
    &&OP_DEPRECATED_9,
    &&OP_DEPRECATED_10,
    &&OP_DEPRECATED_11,
    &&OP_DEPRECATED_12,
    &&OP_getlexref_u,
    &&OP_getlexref_u32,
    &&OP_getlexref_u16,
    &&OP_getlexref_u8,
    &&OP_param_rp_u,
    &&OP_param_op_u,
    &&OP_param_rn_u,
    &&OP_param_on_u,
    &&OP_param_rn2_u,
    &&OP_param_on2_u,
    &&OP_stat_time,
    &&OP_lstat_time,
    &&OP_setdebugtypename,
    &&OP_loadbytecodebuffer,
    &&OP_loadbytecodefh,
    &&OP_throwpayloadlex,
    &&OP_throwpayloadlexcaller,
    &&OP_lastexpayload,
    &&OP_cancelnotify,
    &&OP_decoderconfigure,
    &&OP_decodersetlineseps,
    &&OP_decoderaddbytes,
    &&OP_decodertakechars,
    &&OP_decodertakeallchars,
    &&OP_decodertakeavailablechars,
    &&OP_decodertakeline,
    &&OP_decoderbytesavailable,
    &&OP_decodertakebytes,
    &&OP_decoderempty,
    &&OP_indexingoptimized,
    &&OP_captureinnerlex,
    &&OP_unicmp_s,
    &&OP_DEPRECATED_92,
    &&OP_strfromname,
    &&OP_indexic_s,
    &&OP_getport_sk,
    &&OP_cpucores,
    &&OP_eqaticim_s,
    &&OP_indexicim_s,
    &&OP_decodertakecharseof,
    &&OP_indexim_s,
    &&OP_cas_o,
    &&OP_cas_i,
    &&OP_atomicinc_i,
    &&OP_atomicdec_i,
    &&OP_atomicadd_i,
    &&OP_atomicload_o,
    &&OP_atomicload_i,
    &&OP_atomicstore_o,
    &&OP_atomicstore_i,
    &&OP_barrierfull,
    &&OP_coveragecontrol,
    &&OP_DEPRECATED_111,
    &&OP_DEPRECATED_112,
    &&OP_DEPRECATED_113,
    &&OP_DEPRECATED_114,
    &&OP_DEPRECATED_115,
    &&OP_DEPRECATED_107,
    &&OP_DEPRECATED_108,
    &&OP_DEPRECATED_109,
    &&OP_DEPRECATED_110,
    &&OP_coerce_II,
    &&OP_encoderepconf,
    &&OP_encodeconf,
    &&OP_decodeconf,
    &&OP_decoderepconf,
    &&OP_getppid,
    &&OP_getsignals,
    &&OP_slice,
    &&OP_DEPRECATED_38,
    &&OP_DEPRECATED_39,
    &&OP_DEPRECATED_81,
    &&OP_DEPRECATED_41,
    &&OP_DEPRECATED_42,
    &&OP_DEPRECATED_43,
    &&OP_DEPRECATED_44,
    &&OP_atomicbindattr_o,
    &&OP_casattr_o,
    &&OP_atkey_u,
    &&OP_coerce_us,
    &&OP_DEPRECATED_45,
    &&OP_hllbool,
    &&OP_hllboolfor,
    &&OP_decodelocaltime,
    &&OP_fork,
    &&OP_buffertocu,
    &&OP_writeint,
    &&OP_writeuint,
    &&OP_writenum,
    &&OP_serializetobuf,
    &&OP_readint,
    &&OP_readuint,
    &&OP_readnum,
    &&OP_DEPRECATED_68,
    &&OP_uname,
    &&OP_freemem,
    &&OP_totalmem,
    &&OP_DEPRECATED_93,
    &&OP_DEPRECATED_94,
    &&OP_time,
    &&OP_dispatch_v,
    &&OP_dispatch_i,
    &&OP_dispatch_n,
    &&OP_dispatch_s,
    &&OP_dispatch_o,
    &&OP_gettypehllrole,
    &&OP_sp_guard,
    &&OP_sp_guardconc,
    &&OP_sp_guardtype,
    &&OP_sp_guardhll,
    &&OP_sp_guardsf,
    &&OP_sp_guardsfouter,
    &&OP_sp_guardobj,
    &&OP_sp_guardnotobj,
    &&OP_sp_guardjustconc,
    &&OP_sp_guardjusttype,
    &&OP_sp_guardnonzero,
    &&OP_sp_rebless,
    &&OP_sp_decont,
    &&OP_sp_getlex_o,
    &&OP_sp_getlex_ins,
    &&OP_sp_getlex_no,
    &&OP_sp_bindlex_in,
    &&OP_sp_bindlex_os,
    &&OP_sp_getlexstatic_o,
    &&OP_sp_assertparamcheck,
    &&OP_sp_bindcomplete,
    &&OP_sp_istype,
    &&OP_sp_dispatch_v,
    &&OP_sp_dispatch_i,
    &&OP_sp_dispatch_n,
    &&OP_sp_dispatch_s,
    &&OP_sp_dispatch_o,
    &&OP_sp_getarg_o,
    &&OP_sp_getarg_i,
    &&OP_sp_getarg_n,
    &&OP_sp_getarg_s,
    &&OP_sp_paramnamesused,
    &&OP_sp_getspeshslot,
    &&OP_sp_fastcreate,
    &&OP_sp_get_o,
    &&OP_sp_get_i64,
    &&OP_sp_get_i32,
    &&OP_sp_get_i16,
    &&OP_sp_get_i8,
    &&OP_sp_get_n,
    &&OP_sp_get_s,
    &&OP_sp_bind_o,
    &&OP_sp_bind_i64,
    &&OP_sp_bind_i32,
    &&OP_sp_bind_i16,
    &&OP_sp_bind_i8,
    &&OP_sp_bind_n,
    &&OP_sp_bind_s,
    &&OP_sp_bind_s_nowb,
    &&OP_sp_p6oget_o,
    &&OP_sp_p6ogetvt_o,
    &&OP_sp_p6ogetvc_o,
    &&OP_sp_p6oget_i,
    &&OP_sp_p6oget_n,
    &&OP_sp_p6oget_s,
    &&OP_sp_p6oget_bi,
    &&OP_sp_p6obind_o,
    &&OP_sp_p6obind_i,
    &&OP_sp_p6obind_n,
    &&OP_sp_p6obind_s,
    &&OP_sp_p6oget_i32,
    &&OP_sp_p6obind_i32,
    &&OP_sp_getvt_o,
    &&OP_sp_getvc_o,
    &&OP_sp_fastbox_i,
    &&OP_sp_fastbox_bi,
    &&OP_sp_fastbox_i_ic,
    &&OP_sp_fastbox_bi_ic,
    &&OP_sp_deref_get_i64,
    &&OP_sp_deref_get_n,
    &&OP_sp_deref_bind_i64,
    &&OP_sp_deref_bind_n,
    &&OP_sp_getlexvia_o,
    &&OP_sp_getlexvia_ins,
    &&OP_sp_bindlexvia_os,
    &&OP_sp_bindlexvia_in,
    &&OP_sp_getstringfrom,
    &&OP_sp_getwvalfrom,
    &&OP_sp_jit_enter,
    &&OP_sp_istrue_n,
    &&OP_sp_boolify_iter,
    &&OP_sp_boolify_iter_arr,
    &&OP_sp_boolify_iter_hash,
    &&OP_sp_cas_o,
    &&OP_sp_atomicload_o,
    &&OP_sp_atomicstore_o,
    &&OP_sp_add_I,
    &&OP_sp_sub_I,
    &&OP_sp_mul_I,
    &&OP_sp_bool_I,
    &&OP_sp_runbytecode_v,
    &&OP_sp_runbytecode_i,
    &&OP_sp_runbytecode_n,
    &&OP_sp_runbytecode_s,
    &&OP_sp_runbytecode_o,
    &&OP_sp_runcfunc_v,
    &&OP_sp_runcfunc_i,
    &&OP_sp_runcfunc_n,
    &&OP_sp_runcfunc_s,
    &&OP_sp_runcfunc_o,
    &&OP_sp_runnativecall_v,
    &&OP_sp_runnativecall_i,
    &&OP_sp_runnativecall_n,
    &&OP_sp_runnativecall_s,
    &&OP_sp_runnativecall_o,
    &&OP_sp_resumption,
    &&OP_prof_enter,
    &&OP_prof_enterspesh,
    &&OP_prof_enterinline,
    &&OP_prof_enternative,
    &&OP_prof_exit,
    &&OP_prof_allocated,
    &&OP_prof_replaced,
    &&OP_ctw_check,
    &&OP_coverage_log,
    &&OP_breakpoint,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP,
    &&OP_CALL_EXTOP
};
